##Kommunikationsbeispiel mit der BLE-Bibliothek gatt aus github.com/getsenic/gatt-python
##Diese Bibliothek wird auch mit c't-Artikel von T. Harbaum verwendet.
##Aus dessen Beispielprogramm lego_boost_monitor ist der Quelltext abgeschaut.
##Eigene Klasse MyDeviceManager, um Device Discovery Ergebnis anzuzeigen
##Eine Device Discovery ist bei einem frischen Image nötig, wenn zum ersten Mal mit
##gatt via BLE kommunizert werden soll.
##Programm reagiert in Python-Shell anders als in Bash-Shell: Strg + C beendet
##in Bash Shell nur Device Manager Thread. Daher Abfrage in While-Schleife, ob Device
##Manager Threads noch aktiv. In der Python Shell beendet Strg + C den Hauptthread und
##nachfolgend wird der Device Manager Thread im except Block durch den Hauptthread gestoppt.
##Dieser Bug der Bibliothek gatt kann wie in der Anleitung beschrieben behoben werden.
##8.6.18, S. Mack

import gatt
import threading
import time
import logging

# Nachfolgende Zeile für Debugmeldungen ausschalten (level=10 bedeutet alle Meldungen)
#logging.basicConfig(level=10)

class MyDeviceManager(gatt.DeviceManager):
    def device_discovered(self, device):
        print("Gatt Device Discovery: Device entdeckt [%s] %s" % (device.mac_address, device.alias()))
    # run() muss überschrieben werden mit try/except sonst läuft Thread weiter bei strg + C
    def run(self):
        try:
            super().run()
        except KeyboardInterrupt: # Wird im Fall Bash Shell aufgerufen
            logging.info('Im Device Manager Thread strg + C erkannt: Device Discovery beenden...' )
            self.stop_discovery() # Device Discovery beenden
            time.sleep(1)
            logging.info('Device Manager Thread beenden...')
            self.stop() # Device Manager Thread beenden

print('ACHTUNG: Dieses Programm funktioniert nur unter Linux mit BLE-Kommunikation via BlueZ!')

# MyDeviceManger Instanz erzeugen
manager = MyDeviceManager(adapter_name='hci0')
time.sleep(1)

# Thread für den DeviceMangager        
dman_thread = threading.Thread(target=manager.run,name='devmanager')

print('Device Discovery unter Linux für BlueZ Interface wird gestartet...')
manager.start_discovery()
time.sleep(1)

logging.info('Device Manager starten...')
dman_thread.start() # Hier wirkt try/except nicht bei strg + C   

print('Grünen Taser am Move Hub drücken, damit er als Device erkannt wird!')
print('... signalisiert laufenden Discovery Prozess. Abbruch mit strg + C')

try:
    while True:
        print('.',end='',flush=True) # flush=True damit Ausgabe in Bash Shell
        dman_thread.join(1) # 1 Sekunde lange Warten auf Device Manager Thread
        if not dman_thread.isAlive():
            logging.info('Device Manager Thread wurde beendet. Programm wird abgebrochen.')
            break
except KeyboardInterrupt: # Wird im Fall Python Shell aufgerufen
    logging.info('Im Hauptthread strg + C erkannt: Device Manager Thread wird aus Haupthread heraus beendet...')
    manager.stop_discovery()
    manager.stop()
finally:
    exit(0) # Nur nötig, wenn in Bash Shell gestartet zum beenden Pythoninterpreter
